require 'sketchup'



###################################################################################################
### Class MeshPoint ###############################################################################
###################################################################################################
#
# Point3d	@p		current point
# bool		@is_fixed
# float		@coeff
#
###################################################################################################

class MeshPoint
	attr_accessor :p, :is_fixed, :edge

	def initialize(pt, isfxd, edg)
		@p = pt
		@is_fixed = isfxd
		@edge = edg
	end
end





###################################################################################################
### Class Membrane ################################################################################
###################################################################################################
#
# Point3d[]		@poly		outline
# Point3d[]		@poly2		inner outline
# float[]		@qr			relative tension of the outline
# int			@type		cartesian or not
#
# Selection		@poly_sel
# Group			@group		vertices of the membrane itself
#
# MeshPoint[][]	@m
# flaot			@delta_z
#
###################################################################################################
#
# initialize
# [[Point3d]] get_curves(Selection)
# create_mesh
# create_cartesian_mesh
# create_circ_mesh
# update_edge(int, float)
# swap_vertices(int)
# int outline(Entity)
# simulate
# draw_mesh
# destroy
#
###################################################################################################

class Membrane
attr_reader :type, :group

########## Constructor #############################################################

	def initialize
		s = Sketchup.active_model.selection

		@poly_sel = []
		s.each {|e| @poly_sel.push e;}


		u = get_curve s

		if u == nil
			UI.messagebox "Invalid selection", MB_OK, "FerrariSketch"
			@type = 0
			return
		end

		if u.length > 2
			UI.messagebox "Only one or two curves can be selected", MB_OK, "FerrariSketch"
			@type = 0
			return
		end

		@type = u.length
		@poly = u.shift
		@poly2 = u.shift

		if @type == 1
			@qr = Array.new(@poly.length + 1, -1)
			@qr[-1] = 1.0e25
		else
			@qr = Array.new(@poly.length + @poly2.length, -1)
		end
	end


########## get_curve ###############################################################

	def get_curve(s)

		if s.empty? then return nil; end

		s.each {|e| if e.class != Sketchup::Edge then return nil; end;}

		polys = []

		i = s.length
		while s.length != 0
			if i == s.length
				polys.push []
				polys.last.push s.first.vertices.first.position
				polys.last.push s.first.vertices.last.position
				s.shift
				i = 0
				next
			end

			if s[i].vertices.first.position == polys.last.last
				polys.last.push s[i].vertices.last.position
				s.remove s[i]
				i = 0
				next
			else
				if s[i].vertices.last.position == polys.last.last
					polys.last.push s[i].vertices.first.position
					s.remove s[i]
					i = 0
					next
				end
			end

			i += 1
		end

		polys.each do |e|
			if e.last != e.first
				return nil
			else
				e.shift
			end
		end


		return polys
	end


########## create_mesh #############################################################

	def create_mesh
		if @type == 1
			create_cartesian_mesh
		else
			create_circ_mesh
		end
		draw_mesh
	end


########## create_cartesian_mesh ###################################################

	def create_cartesian_mesh

#rectangle containing the @poly 
		x_min = 1.0/0;
		@poly.each {|e| if e.x < x_min then x_min = e.x; end;}
		x_max = -1.0/0;
		@poly.each {|e| if e.x > x_max then x_max = e.x; end;}
		y_min = 1.0/0;
		@poly.each {|e| if e.y < y_min then y_min = e.y; end;}
		y_max = -1.0/0;
		@poly.each {|e| if e.y > y_max then y_max = e.y; end;}
		z_min = 1.0/0;
		@poly.each {|e| if e.z < z_min then z_min = e.z; end;}
		z_max = -1.0/0;
		@poly.each {|e| if e.z > z_max then z_max = e.z; end;}

		@delta_z = [z_max - z_min, 50].max;

		step = [x_max-x_min, y_max-y_min].min/($m_options["n"]);

		x_min -= step/2;
		x_max += step/2;
		y_min -= step/2;
		y_max += step/2;


		@m = Array.new(1+((x_max-x_min)/step).ceil);
		@m.each_index {|i| @m[i] = Array.new(1+((y_max-y_min)/step).ceil, nil);}

		u = [];									#u[i] : unit vector going from @poly[i] to @poly[i+1]
		lengths = [];							#lengths[i] : length of [@poly[i], @poly[i+1]]
		zm = 0;									#mean altitude
		@poly.each_index do |i|
			t = @poly[i] - @poly[i-1];
			lengths.push Math.sqrt(t.x*t.x + t.y*t.y);
			u.push t.transform(Geom::Transformation.scaling(1/lengths.last));
			zm += @poly[i].z;
		end;
		u.push u.shift;
		lengths.push lengths.shift;
		zm /= @poly.length;


		@m.each_index do |i|
			@m.first.each_index do |j|
				p = Geom::Point3d.new(x_min + i*step, y_min + j*step, zm);		#current point
				if Geom.point_in_polygon_2D(p, @poly, true)
					@m[i][j] = MeshPoint.new(p, false, -1)
				else
					nearest_index = nil;
					nearest_dist = 1.0/0;

					@poly.each_index do |k|
						p2 = p - @poly[k]
						t = (p2.x*u[k].y - u[k].x*p2.y).abs
						if t < nearest_dist
							t2 = p2.x*u[k].x + u[k].y*p2.y
							if (0 <= t2) and (t2 <= lengths[k])
								nearest_index = k
								nearest_dist = t
							end
						end
					end

					@poly.each_index do |k|
						t = p - @poly[k]
						t.z = 0
						if t.length < nearest_dist
							nearest_dist = t.length
							nearest_index = k
						end
					end
					
					@m[i][j] = MeshPoint.new(p, false, nearest_index) if nearest_dist < step
				end
			end
		end
		@m.push Array.new(@m.first.length, nil)

		ral = []
		@m.each_index do |i|
			next if i == @m.length-1
			@m.first.each_index do |j|
				next if @m[i][j] == nil
				ngbh = 0;
				if j>0
					ngbh+=1 if @m[i-1][j-1] != nil
					ngbh+=1 if @m[i][j-1] != nil
					ngbh+=1 if @m[i+1][j-1] != nil
					ngbh+=1 if @m[i-1][j] != nil
					ngbh+=1 if @m[i+1][j] != nil
					ngbh+=1 if @m[i-1][j+1] != nil
					ngbh+=1 if @m[i][j+1] != nil
					ngbh+=1 if @m[i+1][j+1] != nil
				end

				if ngbh == 8
					@m[i][j].edge = -1
					next
				end

				ral.push @m[i][j]
				if @m[i][j].edge == -1
					p = @m[i][j].p
					nearest_index = nil;
					nearest_dist = 1.0/0;

					@poly.each_index do |k|
						p2 = p - @poly[k]
						t = (p2.x*u[k].y - u[k].x*p2.y).abs
						if t < nearest_dist
							t2 = p2.x*u[k].x + u[k].y*p2.y
							if (0 <= t2) and (t2 <= lengths[k])
								nearest_index = k
								nearest_dist = t
							end
						end
					end

					@poly.each_index do |k|
						t = p - @poly[k]
						t.z = 0
						if t.length < nearest_dist
							nearest_dist = t.length
							nearest_index = k
						end
					end
					@m[i][j].edge = nearest_index
				end
			end
		end

		#ral.each {|n| Sketchup.active_model.entities.add_face n.p, n.p+Geom::Vector3d.new(1, 0, 10*(1+n.edge)), n.p+Geom::Vector3d.new(-1, 0, 10*(1+n.edge));}
=begin
		order = []
		@poly.each_index do |i|
			order.push [(Math::PI/2-(@poly[i-2] - @poly[i-1]).angle_between(@poly[i-1] - @poly[i])).abs, i-1]
		end;
		order.sort!
		order.each_index {|i| order[i] = order[i].last;}
=end
		@poly.each_index do |k|
			i0 = ((@poly[k].x - x_min)/step).floor
			j0 = ((@poly[k].y - y_min)/step).floor
			nearest_dist = 1.0/0
			nearest_ref = nil

			ral.each do |r|
				d = r.p - @poly[k]
				d.z = 0;
				t = d.length;
				if t < nearest_dist
					nearest_dist = t
					nearest_ref = r
				end
			end

			if nearest_ref != nil
				nearest_ref.p = @poly[k]
				nearest_ref.is_fixed = true
				nearest_ref.edge = @poly.length
				ral.delete nearest_ref
			end
		end
	end


########## create_circ_mesh ########################################################

	def create_circ_mesh

		g = Geom::Point3d.new 0, 0, 0
		###g = [Geom::Point3d.new 0, 0, 0]
		@poly.each {|p| g.last.x+=p.x;g.last.y+=p.y;g.last.z+=p.z;}
		g.last.x/=@poly.length;g.last.y/=@poly.length;g.last.z/=@poly.length
		g.push Geom::Point3d.new(0, 0, 0)
		@poly2.each {|p| g.last.x+=p.x;g.last.y+=p.y;g.last.z+=p.z;}
		g.last.x/=@poly2.length;g.last.y/=@poly2.length;g.last.z/=@poly2.length

		if g.last.z>g.first.z
			t = @poly
			@poly = @poly2
			@poly2 = t
			g.push g.shift
		end

		@delta_z = [g.first.z-g.last.z, 50].max

		angular_step = 2*Math::PI/$m_options["n2"]

		l = []
		polys = [@poly, @poly2]
		polys.each_index do |i| # liste des rayons ncessaire pour attacher le maillage
			poly = polys[i]
			t = []
			poly.each do |s|
				d = s - g[i]
				d.z = 0
				angle = Math::acos(d.x/d.length)
				if d.y < 0 then angle = -angle; end
				t.push angle
			end

			t0 = t.first # angle du sommet 0
			dir = 0
			t.each_index {|j| dir += 1 if t[j-1] < t[j];} # sens de parcours du contour
			t.sort!
			t.reverse! if 2*dir < poly.length
			while t.first != t0 do t.push t.shift end

			if i == 0 then t.each_index {|j| l.push [t[j], j, -1];}
			else t.each_index {|j| l.push [t[j], -1, j];} end
		end
		l.sort!
		

		i = l.length - 1
		while i>=0 do # fusion si possible de certains rayons
			if ((l[i].first - l[i-1].first) % (2*Math::PI)) <= angular_step
				if (l[i-1][2] == -1) and (l[i][1] == -1)
					l[i-1][2] = l[i][2]
					l.delete_at i
					i -= 1
				elsif (l[i-1][1] == -1) and (l[i][2] == -1)
					l[i-1][1] = l[i][1]
					l.delete_at i
					i -= 1
				end
			end
			i -= 1
		end


		l.each_index do |li| # placement de la seconde extrmit de chaque rayon
			angle, i1, i2 = l[li]
			if i1 == -1 then i = 0
			elsif i2 == -1 then i = 1
			else
				l[li][1] = MeshPoint.new @poly[i1], true, i1
				l[li][2] = MeshPoint.new @poly2[i2], true, @poly.length+i2
				next
			end

			poly = polys[i]
			t = 0
			edg = -2
			c = Math.cos(angle)
			s = Math.sin(angle)

			poly.each_index do |j|
				ab = poly[j] - poly[j-1]
				ag = g[i] - poly[j-1]
				d = ab.x * s - ab.y * c

				if d == 0 then next; end

				t = (ag.x*s - ag.y*c)/d
				if t<0 or t>1 then next; end

				r = (ab.x*ag.y - ab.y*ag.x)/d
				if r <= 0
					edg = j-1
					break
				end
			end

			if i==0
				l[li][1] = MeshPoint.new Geom::Point3d.linear_combination(1-t, poly[edg], t, poly[edg+1]), false, (edg%@poly.length)
				l[li][2] = MeshPoint.new @poly2[i2], true, @poly.length+i2
			else
				l[li][1] = MeshPoint.new @poly[i1], true, i1
				l[li][2] = MeshPoint.new Geom::Point3d.linear_combination(1-t, poly[edg], t, poly[edg+1]), false, @poly.length+(edg%@poly2.length)
			end
		end

		@m = []
		n1 = $m_options["n1"]
		rests = []
		l.each_index do |li| # remplissage des plages angulaires trop grandes
			p11 = l[li-1][1]
			p12 = l[li-1][2]
			p21 = l[li][1]
			p22 = l[li][2]

			t = ((l[li].first - l[li-1].first) % (2*Math::PI))/angular_step
			nb_ray = [1, t.round].max
			t -= t.floor.to_f
			rests.push [t, @m.length]
			(1..nb_ray).to_a.each do |i|
				pp1 = Geom::Point3d.linear_combination((nb_ray-i).to_f/nb_ray, p11.p, i.to_f/nb_ray, p21.p)
				pp2 = Geom::Point3d.linear_combination((nb_ray-i).to_f/nb_ray, p12.p, i.to_f/nb_ray, p22.p)
				@m.push []
				(0..n1-1).to_a.each do |j|
					@m.last.push MeshPoint.new(Geom::Point3d.linear_combination((n1-1-j).to_f/(n1-1), pp1, j.to_f/(n1-1), pp2), false, -1)
				end
				@m.last.first.edge = l[li][1].edge
				@m.last.last.edge = l[li][2].edge
			end
			@m.last.first.is_fixed = l[li][1].is_fixed
			@m.last.last.is_fixed = l[li][2].is_fixed
		end

		rests.reverse!
		rests = rests.sort.reverse
		while @m.length < $m_options["n2"]
			i = rests.pop.last
			@m.insert i, []
			(0..n1-1).to_a.each {|j| @m[i][j] = @m[i-1][j].clone;}
			@m[i].first.is_fixed = false
			@m[i].last.is_fixed = false
			@m[i].first.edge = [@m[i-1].first.edge, ((i+1<@m.length)? @m[i+1].first.edge : @m[0].first.edge)].min
			@m[i].last.edge = [@m[i-1].last.edge, ((i+1<@m.length)? @m[i+1].last.edge : @m[0].last.edge)].min
		end

		@m.reverse!
	end


########## update_edge #############################################################

	def update_edge(k, t)
		@qr[k] = t
	end


########## swap_vertices ###########################################################

	def swap_vertices(k)
		if k < @poly.length
			p1 = @poly[k]
			p2 = @poly[k-@poly.length+1]
		else
			k -= @poly.length
			p1 = @poly2[k]
			p2 = @poly[k-@poly2.length+1]
		end

		pm1 = nil
		pm2 = nil
		@m.each do |m2|
			m2.each do |pm|
				next if pm == nil
				next if not pm.is_fixed
				pm1 = pm if pm.p == p1
				pm2 = pm if pm.p == p2
			end
		end

		pm1.p, pm2.p = [pm2.p, pm1.p]
	end


########## outline #################################################################

	def outline(e)
		pt = e.vertices.first.position
		pt2 = e.vertices.last.position

		i = @poly.index pt
		if i != nil
			return @poly.length-1 if (i==0) and (@poly.last == pt2)
			return @poly.length-1 if (i==@poly.length-1) and (@poly.first == pt2)
			j = @poly.index pt2
			return [i, j].min if (i-j).abs == 1
		end

		if @poly2 == nil
			return -1
		end

		i = @poly2.index pt
		if i != nil
			return @poly.length+@poly2.length-1 if (i==0) and (@poly2.last == pt2)
			return @poly.length+@poly2.length-1if (i==@poly2.length-1) and (@poly2.first == pt2)
			j = @poly2.index pt2
			return (@poly.length+[i, j].min) if (i-j).abs == 1
		end

		return -1
	end


########## simulate ################################################################

	def simulate
		@qr.each_index {|i| if @qr[i] == -1 then @qr[i] = $m_options["qr"]; end;}

		epsilon = @delta_z * 10 ** (-$m_options["precision"])
		infl = $m_options["infl"]

		l1 = @m.length-2
		l2 = @m.first.length-1


		if @type == 1
			q1 = $m_options["qc"] * Math::cos($m_options["theta"]).abs ** 2 + Math::sin($m_options["theta"]).abs ** 2
			q2 = $m_options["qc"] * Math::sin($m_options["theta"]).abs ** 2 + Math::cos($m_options["theta"]).abs ** 2
		else
			q1 = 1
			q2 = $m_options["qc"]
		end

		l = []
		begin
			stop = true

			@m.each_index do |i|
				m = @m.at(i-1)
				m.each_index do |j|
					next if m.at(j) == nil
					next if m.at(j).is_fixed
					l.push [i-1, j]
				end
			end

			begin
				i, j = l.shift

				p = Geom::Point3d.new 0, 0, 0
				div = 0.0
				ngbh = ((infl == 0.0)? -1 : 0)									# if infl == 0, ngbh will never reach 4

				r = (@m.at(i).at(j).edge != -1)

				o1 = @m.at(i-1).at(j)
				if o1 != nil
					coeff = (r and (o1.edge != -1))? @qr[o1.edge] : q1
					o1 = o1.p

					p.x += coeff * o1.x
					p.y += coeff * o1.y
					p.z += coeff * o1.z
					div += coeff

					ngbh += 1
				end

				if (o2 = @m.at(i).at(j+1)) != nil
					coeff = (r and (o2.edge != -1))? @qr[o2.edge] : q2
					o2 = o2.p

					p.x += coeff * o2.x
					p.y += coeff * o2.y
					p.z += coeff * o2.z
					div += coeff

					ngbh += 1
				end

				o3 = @m.at(i+1).at(j)
				if o3 != nil
					coeff = (r and (o3.edge != -1))? @qr[o3.edge] : q1
					o3 = o3.p

					p.x += coeff * o3.x
					p.y += coeff * o3.y
					p.z += coeff * o3.z
					div += coeff

					ngbh += 1
				end

				if (j > 0) and ((o4 = @m.at(i).at(j-1)) != nil)
					coeff = (r and (o4.edge != -1))? @qr[o4.edge] : q2
					o4 = o4.p

					p.x += coeff * o4.x
					p.y += coeff * o4.y
					p.z += coeff * o4.z
					div += coeff

					ngbh += 1
				end

				if ngbh == 4
					n = (o3-o1).cross(o2-o4)
					q = infl/Math.sqrt(n.length)
					n.normalize!

					p.x += q*n.x
					p.y += q*n.y
					p.z += q*n.z
				end

				p.x /= div
				p.y /= div
				p.z /= div

				if (p - @m.at(i).at(j).p).length > epsilon
					l.push [i, j]
					stop = false
				end

				@m.at(i).at(j).p = p
			end while not l.empty?

		end while not stop

		draw_mesh
	end


########## draw_mesh ###############################################################

	def draw_mesh
		@group = Sketchup.active_model.entities.add_group if @group.class != Sketchup::Group or @group.deleted?

		if @group.entities.count != 0
			destroy
		end
		ent = Sketchup.active_model.entities

		mesh = Geom::PolygonMesh.new

		for i in -1..@m.length-2
			for j in 0..@m.first.length-2
				if (@m[i+1][j] != nil) and (@m[i][j+1] != nil)
					mesh.add_polygon(@m[i][j].p, @m[i+1][j].p, @m[i][j+1].p) if @m[i][j] != nil
					mesh.add_polygon(@m[i+1][j].p, @m[i+1][j+1].p, @m[i][j+1].p) if @m[i+1][j+1] != nil
				elsif (@m[i][j] != nil) and (@m[i+1][j+1] != nil)
					mesh.add_polygon(@m[i][j].p, @m[i+1][j].p, @m[i+1][j+1].p) if @m[i+1][j] != nil
					mesh.add_polygon(@m[i][j].p, @m[i+1][j+1].p, @m[i][j+1].p) if @m[i][j+1] != nil
				end
			end
		end

		@group.entities.add_faces_from_mesh mesh

	end


########## destroy #################################################################

	def destroy
		@group.entities.clear!

		Sketchup.active_model.entities.add_curve @poly
		Sketchup.active_model.entities.add_edges @poly[-1], @poly[0]
		if @type == 2
			Sketchup.active_model.entities.add_curve @poly2
			Sketchup.active_model.entities.add_edges @poly2[-1], @poly2[0]
		end
	end
end





###################################################################################################
### Global variables ##############################################################################
###################################################################################################

$membranes	= []

$cur_m = false

$m_options = {
	"n" => 20,
	"n1" => 12,
	"n2" => 36,
	"precision" => 3.0,
	"qr" => 25.0,
	"qc" => 1.0,
	"infl" => 0.0,
	"theta" => 0.0,
}




###################################################################################################
### Toolbar #######################################################################################
###################################################################################################

def ferrariSketch_menu_select
	i=$membranes.length-1
	while i>=0
		$membranes.delete_at i if $membranes[i].group.deleted?
		i-=1
	end


	$membranes.unshift Membrane.new
	t = $membranes.first.type
	err = true
 
	if t == 1
		$m_options["theta"] = 180 * $m_options["theta"]/Math::PI
   		results = UI.inputbox	["Inflating pressure ", 	"Warp/Weft tension ratio ", 	"Warp orientation ", 	"Default edge cable tension ", 	"Subdivisions ", 	"Accuracy level "],
								[$m_options["infl"],			$m_options["qc"], 				$m_options["theta"], 		$m_options["qr"], 				$m_options["n"], 	$m_options["precision"]],
								"FerrariSketch - Create textile structure: options"
 		if results != false
     		$m_options["infl"], $m_options["qc"], $m_options["theta"], $m_options["qr"], $m_options["n"], $m_options["precision"] = results
      		$m_options["theta"] = Math::PI * $m_options["theta"]/180
      		err = false
    	end
  	end

	if t == 2
		results = UI.inputbox ["Inflating pressure ", "Warp/Weft tension ratio ", "Default edge cable tension ", "Radial subdivisions ", "Angular subdivisions", "Accuracy level "], [$m_options["infl"], $m_options["qc"], $m_options["qr"], $m_options["n1"], $m_options["n2"], $m_options["precision"]], "FerrariSketch - Create textile structure: options"
		if results != false
			$m_options["infl"], $m_options["qc"], $m_options["qr"], $m_options["n"], $m_options["n2"], $m_options["precision"] = results
			err = false
   		end
  	end

	$m_options["n"] = $m_options["n"].to_i
	$m_options["n1"] = $m_options["n1"].to_i
	$m_options["n2"] = $m_options["n2"].to_i
	$m_options["precision"] = $m_options["precision"].to_f
	$m_options["qr"] = $m_options["qr"].to_f
	$m_options["theta"] = $m_options["theta"].to_f
	$m_options["qc"] = $m_options["qc"].to_f
	$m_options["infl"] = $m_options["infl"].to_f

	$m_options["n"] = 5 if $m_options["n"] < 5
	$m_options["n1"] = 5 if $m_options["n1"] < 5
	$m_options["n2"] = 5 if $m_options["n2"] < 5
	$m_options["precision"] = 1 if $m_options["precision"] < 1.0
	$m_options["qr"] = 0.1 if $m_options["qr"] <= 0.0
	$m_options["qc"] = 0.1 if $m_options["qc"] <= 0.0

	if err
		$membranes.shift
		$cur_m = false
	else
		$cur_m = true
		$membranes.first.create_mesh
	end
end

####################################################################################


def ferrariSketch_menu_selectall
	s = Sketchup.active_model.selection
	if ((s.length != 1) or ($membranes.length == 0))
		UI.beep
		return
	end

	g = s[0]
	found = -1
	for i in 0..$membranes.length-1
		if($membranes[i].group == g)
			found = i
			break
		end
	end

	if found != -1
		$membranes.unshift $membranes.delete_at found
		$cur_m = true
	else
		UI.messagebox "Invalid selection", MB_OK, "FerrariSketch"
	end

	s.clear
end


####################################################################################

def ferrariSketch_menu_tension
	s = Sketchup.active_model.selection

	s.each {|e| s.remove(e) if e.class != Sketchup::Edge;}

	if s.length == 0
		UI.messagebox "Invalid selection", MB_OK, "FerrariSketch"
		return
	end

	result = UI.inputbox ["Cable tension       ", "Straight Edge?       "], [$m_options["qr"], "No"], ["", "Yes|No"], "FerrariSketch - Modify cable curvature"
	if result == false
		return
	end
	result = ((result.last == "Yes")? 1.0e20 : result.first)
	s.each do |e|
		k = $membranes.first.outline(e)
		$membranes.first.update_edge(k, result) if k != -1
	end
	s.clear
end


####################################################################################

def ferrariSketch_menu_swap
	s = Sketchup.active_model.selection

	s.each {|e| s.remove(e) if e.class != Sketchup::Edge;}

	if s.length == 0
		UI.messagebox "Invalid selection", MB_OK, "FerrariSketch"
		return
	end

	s.each do |e|
		k = $membranes.first.outline(e)
		$membranes.first.swap_vertices(k) if k != -1
	end

	s.clear
	$membranes.first.draw_mesh
end


####################################################################################

def ferrariSketch_menu_options
	t = $membranes.first.type
 
	if t == 1
		$m_options["theta"] = 180 * $m_options["theta"]/Math::PI
  		results = UI.inputbox ["Inflating pressure ", "Warp/weft tension ratio ", "Warp orientation ", "Accuracy level "], [$m_options["infl"], $m_options["qc"], $m_options["theta"], $m_options["precision"]], "FerrariSketch - Modify fabric membrane"
   		if results != false
      		$m_options["infl"], $m_options["qc"], $m_options["theta"], $m_options["precision"] = results
      		$m_options["theta"] = Math::PI * $m_options["theta"]/180

			$m_options["precision"] = $m_options["precision"].to_f
			$m_options["theta"] = $m_options["theta"].to_f
			$m_options["qc"] = $m_options["qc"].to_f
			$m_options["infl"] = $m_options["infl"].to_f

			$m_options["precision"]= 1  if $m_options["precision"] < 1
			$m_options["qc"] = 0.1 if $m_options["qc"] <= 0

      		err = false
    		end
  	end

	if t == 2
  		results = UI.inputbox ["Inflating pressure ", "Warp/weft tension ratio ", "Accuracy level "], [$m_options["infl"], $m_options["qc"], $m_options["precision"]], "FerrariSketch - Modify fabric membrane"
  		if results != false
      		$m_options["infl"], $m_options["qc"], $m_options["precision"] = results

			$m_options["precision"] = $m_options["precision"].to_f
			$m_options["qc"] = $m_options["qc"].to_f
			$m_options["infl"] = $m_options["infl"].to_f

			$m_options["precision"]= 1  if $m_options["precision"] < 1
			$m_options["qc"] = 0.1 if $m_options["qc"] <= 0
     		err = false
    		end
 	end

	if $m_options["precision"]<1 then $m_options["precision"]=1; end
	if $m_options["qc"]<=0 then $m_options["qc"]=0.1; end
end


####################################################################################

def ferrariSketch_menu_run
	$cur_m = false
	$membranes.first.simulate
end


####################################################################################

def ferrariSketch_menu_destroy
	if($cur_m)
		$cur_m = false
		$membranes.shift
	else
		s = Sketchup.active_model.selection
		if ((s.length != 1) or ($membranes.length == 0))
			UI.beep
			return
		end

		g = s[0]
		found = -1
		for i in 0..$membranes.length-1
			if($membranes[i].group == g)
				found = i
				break
			end
		end

		if found != -1
			$membranes[found].destroy
			$membranes.delete_at found
			$cur_m = false
		else
			UI.messagebox "Invalid selection", MB_OK, "FerrariSketch"
		end

		s.clear
	end
end

####################################################################################
####################################################################################
